@@ -1,7 +1,7 @@
 package Config::IOD::Reader;
 
-our $DATE = '2015-03-12'; # DATE
-our $VERSION = '0.08'; # VERSION
+our $DATE = '2015-03-13'; # DATE
+our $VERSION = '0.09'; # VERSION
 
 use 5.010001;
 use strict;
@@ -133,75 +133,15 @@ sub _read_string {
             my $name = $1;
             my $val  = $2;
 
-            my $enc;
-            my $impenc; # implicit encoding
-            if ($self->{enable_encoding} && $val =~ /^!(\w+) (.*)/) {
-                $enc = $1;
-                $val = $2;
-            } elsif ($self->{enable_quoting} && $val =~ /^"/) {
-                $val =~ s/("[^"]*")\s*[;#].*/$1/; # strip comment (not perfect)
-                my $res = $self->_decode_json($val);
-                if ($res->[0] != 200) {
-                    $self->_err("Invalid JSON string");
-                }
-                $val = $res->[2];
-                $impenc++;
-            } elsif ($self->{enable_bracket} && $val =~ /^\[/) {
-                $val =~ s/(\[[^\]]*\])\s*[;#].*/$1/; # strip comment (not perfect)
-                my $res = $self->_decode_json($val);
-                if ($res->[0] != 200) {
-                    $self->_err("Invalid JSON array");
-                }
-                $val = $res->[2];
-                $impenc++;
-            } elsif ($self->{enable_brace} && $val =~ /^\{/) {
-                $val =~ s/(\{[^\]]*\})\s*[;#].*/$1/; # strip comment (not perfect)
-                my $res = $self->_decode_json($val);
-                if ($res->[0] != 200) {
-                    $self->_err("Invalid JSON object (hash)");
-                }
-                $val = $res->[2];
-                $impenc++;
-            }
-
-            if (defined $enc) {
-                # canonicalize shorthand
-                $enc = "json" if $enc eq 'j';
-                $enc = "hex"  if $enc eq 'h';
-                $enc = "expr" if $enc eq 'e';
-                if ($self->{allow_encodings}) {
-                    $self->_err("Encoding '$enc' is not in ".
-                                    "allow_encodings list")
-                        unless grep { $_ eq $enc } @{$self->{allow_encodings}};
-                }
-                if ($self->{disallow_encodings}) {
-                    $self->_err("Encoding '$enc' is in ".
-                                    "disallow_encodings list")
-                        if grep { $_ eq $enc } @{$self->{disallow_encodings}};
-                }
-                if ($enc eq 'json') {
-                    my $res = $self->_decode_json($val);
-                    if ($res->[0] != 200) {
-                        $self->_err("Invalid JSON");
-                    }
-                    $val = $res->[2];
-                } elsif ($enc eq 'hex') {
-                    $val =~ s/\s*[;#].*\z//; # shave comment
-                    $val = $self->_decode_hex($val);
-                } elsif ($enc eq 'base64') {
-                    $val =~ s/\s*[;#].*\z//; # shave comment
-                    $val = $self->_decode_base64($val);
-                } elsif ($enc eq 'expr') {
-                    $self->_err("Expr is not allowed (enable_expr=0)")
-                        unless $self->{enable_expr};
-                    $val = $self->_decode_expr($val);
-                } else {
-                    $self->_err("Unknown encoding '$enc'");
-                }
+            # the common case is that value are not decoded or
+            # quoted/bracketed/braced, so we avoid calling _parse_raw_value here
+            # to avoid overhead
+            if ($val =~ /\A["!\\[\{]/) {
+                my ($err, $parse_res, $decoded_val) = $self->_parse_raw_value($val);
+                $self->_err("Invalid value: " . $err) if $err;
+                $val = $decoded_val;
             } else {
-                unless ($impenc) {
-                    $val =~ s/\s*[;#].*\z//; # shave comment
-                }
+                $val =~ s/\s*[#;].*//; # strip comment
             }
 
             if (exists $res->{$cur_section}{$name}) {
@@ -243,7 +183,7 @@ Config::IOD::Reader - Read IOD configuration files
 
 =head1 VERSION
 
-This document describes version 0.08 of Config::IOD::Reader (from Perl distribution Config-IOD-Reader), released on 2015-03-12.
+This document describes version 0.09 of Config::IOD::Reader (from Perl distribution Config-IOD-Reader), released on 2015-03-13.
 
 =head1 SYNOPSIS
 
